/*
	ph_.h ver 1.4 (c) PaulHoule.com  10/6/2019

	Tiny replacement for stdlib (32 or 64-bit) -- #include this header and
	link with ph_.lib.  Targets are simple x86 PC command line tools.
	Designed for X86 Windows (32 or 64-bit) and Microsoft C compiler.

	Creates smaller .exe.  Also eliminates dependencies -- only the Windows
	DLL's are required (msvcrt.dll is not required).  Speeds up compilation
	since only ph_.h is included, and ph_.h includes no other file.

	A program really needs to be written with the target of using this
	library -- stdlib use must be appropriately restricted.

	The interface is compatible with (though a small subset of) stdlib,
	so a program can be built either way for testing, eg:

		#ifdef PH_
		  #include <ph_.h>
		#else
		  #include <stdlib.h>
		  #include <stdio.h>
		  ...etc
		#endif
	
	Some drawbacks (there are many more than these):
	 *)	A small subset of stdlib is implemented.
	 *) Command line arguments (argc, argv) are not supported -- must
		get command line via Windows GetCommandLine() call.
	 *)	Library is not thread-safe.
	 *) Reading file/stream in text mode is unsupported (yields zero bytes).
		Must fopen(, "rb") or _setmode(, _O_BINARY) to read.
		However text mode write ("\n" --> "\r\n") is supported.
	 *) Fixed file table -- only a dozen fopen'd FILE * handles at a time.
	 *)	What is implemented is only partially functional.  As an example,
		printf() flavors all use the Windows user32 wvsprintf() which:
		(1) doesn't support floats; (2) limits the output to 1024 chars;
		(3) no support for %ll qualifier (MSFT-specific I64 must be used).
*/

#ifndef PH_H						// include guard
#define PH_H
#ifdef __cplusplus
extern "C" {
#endif

#define _fileno(h)	((int) (intptr_t) (h))

#ifdef _WIN64
  typedef unsigned __int64 size_t;
  typedef __int64 intptr_t;
  typedef __int64 ptrdiff_t;
#else
  typedef unsigned __int32 size_t;
  typedef __int32 intptr_t;
  typedef __int32 ptrdiff_t;
#endif
typedef __int8  int8_t;
typedef __int16 int16_t;
typedef __int32 int32_t;
typedef __int64 int64_t;
typedef unsigned __int8  uint8_t;
typedef unsigned __int16 uint16_t;
typedef unsigned __int32 uint32_t;
typedef unsigned __int64 uint64_t;
typedef unsigned __int16 wchar_t;
typedef unsigned __int16 __ph_uwchar_t;
typedef unsigned __int16 wint_t;
typedef __int8 *va_list;

// va_ stuff is wrong for 64-bit struct args
#define __ph_vasz(t) (sizeof(t)+sizeof(size_t)-1 & ~(sizeof(size_t)-1))
#define va_start(vlv, pV) ( vlv = (va_list) &pV + __ph_vasz(pV) )
#define va_end(vlv)
#define NULL ((void *) 0)
typedef struct xfile FILE;
#define EOF (-1)
#define _O_TEXT 0
#define _O_BINARY 8
#define SEEK_SET 0
#define SEEK_CUR 1
#define SEEK_END 2
#define stdin  ((void *) 1)
#define stdout ((void *) 2)
#define stderr ((void *) 3)

void __fastcall exit(unsigned ecode);

int __fastcall printf(  char   *pFmt, ...);
int __fastcall wprintf(wchar_t *pFmt, ...);
int __fastcall vprintf(  char   *pFmt, va_list vl);
int __fastcall vwprintf(wchar_t *pFmt, va_list vl);
int __fastcall fprintf( FILE *stream,  char   *pFmt, ...);
int __fastcall fwprintf(FILE *stream, wchar_t *pFmt, ...);
int __fastcall snprintf( char   *pDst, size_t n,  char   *pFmt, ...);
int __fastcall swprintf(wchar_t *pDst, size_t n, wchar_t *pFmt, ...);
int __fastcall vsnprintf( char   *pDst, size_t n,  char   *pFmt, va_list vl);
int __fastcall vswprintf(wchar_t *pDst, size_t n, wchar_t *pFmt, va_list vl);
int __fastcall vfprintf( FILE *stream,  char   *pFmt, va_list vl);
int __fastcall vfwprintf(FILE *stream, wchar_t *pFmt, va_list vl);

int __fastcall sprintf(char *pDst, char *pFmt, ...);
int __fastcall vsprintf(char *pDst, char *pFmt, va_list vl);

int __fastcall fputc(int c, FILE *stream);
#define putc fputc
int __fastcall putchar(int c);
int __fastcall fputs(char *p, FILE *stream);
int __fastcall puts(char *p);

void *__fastcall malloc(size_t n);
void __fastcall free(void *p);

int __fastcall isspace(int c);
int __fastcall iswspace(wint_t c);
int __fastcall isdigit(int c);
int __fastcall iswdigit(wint_t c);
int __fastcall isalpha(int c);
int __fastcall iswalpha(wint_t c);
int __fastcall islower(int c);
int __fastcall iswlower(wint_t c);
int __fastcall isupper(int c);
int __fastcall iswupper(wint_t c);

int __fastcall memcmp(void *pDst, void *pSrc, size_t n);
int __fastcall wmemcmp(wchar_t *pDst, wchar_t *pSrc, size_t n);
#if 0 // 9/24/2019: There is no memcpy any longer; memmove is used
void *__fastcall memcpy(void *pDst, void *pSrc, size_t n);
wchar_t *__fastcall wmemcpy(wchar_t *pDst, wchar_t *pSrc, size_t n);
#endif
#define memcpy memmove
#define wmemcpy wmemmove
void *__fastcall memmove(void *pDst, void *pSrc, size_t n);
wchar_t *__fastcall wmemmove(wchar_t *pDst, wchar_t *pSrc, size_t n);
void *__fastcall memset(void *pDst, int c, size_t n);
wchar_t *__fastcall wmemset(wchar_t *pDst, wchar_t c, size_t n);
void *__fastcall memchr(void *pDst, int c, size_t n);
wchar_t *__fastcall wmemchr(wchar_t *ws, wchar_t c, size_t n);
size_t __fastcall strlen(char *pSrc);
size_t __fastcall wcslen(wchar_t *pSrc);
char *__fastcall strcpy(char *pDst, char *pSrc);
wchar_t *__fastcall wcscpy(wchar_t *pDst, wchar_t *pSrc);
char *__fastcall strncpy(char *pDst, char *pSrc, size_t n);
wchar_t *__fastcall wcsncpy(wchar_t *pDst, wchar_t *pSrc, size_t n);
int __fastcall strcmp(char *pDst, char *pSrc);
int __fastcall wcscmp(wchar_t *pDst, wchar_t *pSrc);
int __fastcall strncmp(char *pDst, char *pSrc, size_t n);
int __fastcall wcsncmp(wchar_t *pDst, wchar_t *pSrc, size_t n);
char *__fastcall strcat(char *pDst, char *pSrc);
wchar_t *__fastcall wcscat(wchar_t *pDst, wchar_t *pSrc);
char *__fastcall strncat(char *pDst, char *pSrc, size_t n);
wchar_t *__fastcall wcsncat(wchar_t *pDst, wchar_t *pSrc, size_t n);
char *__fastcall strchr(char *pSrc, int c);
wchar_t *__fastcall wcschr(wchar_t *pSrc, wchar_t c);
char *__fastcall strrchr(char *pSrc, int c);
wchar_t *__fastcall wcsrchr(wchar_t *pSrc, wchar_t c);
char *__fastcall strchr(char *pSrc, int c);
wchar_t *__fastcall wcschr(wchar_t *pSrc, wchar_t c);
char *__fastcall strstr(char *pDst, char *pSrc);
wchar_t *__fastcall wcsstr(wchar_t *pDst, wchar_t *pSrc);

int __fastcall _setmode(int fn, int mode);
void __fastcall clearerr(FILE *stream);
int __fastcall ferror(FILE *stream);
int __fastcall feof(FILE *stream);
FILE * __fastcall fopen(char *fn, char *m);
FILE * __fastcall _wfopen(wchar_t *fn, wchar_t *m);
int __fastcall fclose(FILE *stream);
size_t __fastcall fread(void *p, size_t sz, size_t n, FILE *stream);
size_t __fastcall fwrite(void *p, size_t sz, size_t n, FILE *stream);
int __fastcall fseek(FILE *stream, long off, int orig);
long __fastcall ftell(FILE *stream);

#ifdef __cplusplus
}
#endif
#endif									// include guard
